@@ -5,9 +5,9 @@ mod error;
 
 pub use crate::error::{TreeError, TreeResult};
 use deltoid::{Apply, Core, Delta, DeltaError, DeltaResult, FromDelta, IntoDelta};
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use serde::de::{self, MapAccess, SeqAccess, Visitor};
 use serde::ser::SerializeStruct;
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use serde_derive::{Deserialize, Serialize};
 use std::collections::{HashMap, VecDeque};
 use std::fmt::{self, Debug};
@@ -19,9 +19,14 @@ pub struct ArenaTree<D: Clone + Debug + Default + PartialEq> {
 }
 
 #[allow(unused)]
-impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
+impl<D> ArenaTree<D>
+where
+    D: Clone + Debug + Default + PartialEq,
+{
     #[inline(always)]
-    pub fn new() -> Self { Self::with_capacity(64) }
+    pub fn new() -> Self {
+        Self::with_capacity(64)
+    }
 
     #[inline(always)]
     pub fn with_capacity(node_count: usize) -> Self {
@@ -33,16 +38,22 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
 
     #[inline(always)]
     pub fn node_ref(&self, idx: NodeIdx) -> TreeResult<&Node<D>> {
-        self.nodes.get(idx.0).ok_or(TreeError::NoNodeForNodeIdx { idx })
+        self.nodes
+            .get(idx.0)
+            .ok_or(TreeError::NoNodeForNodeIdx { idx })
     }
 
     #[inline(always)]
     pub fn node_mut(&mut self, idx: NodeIdx) -> TreeResult<&mut Node<D>> {
-        self.nodes.get_mut(idx.0).ok_or(TreeError::NoNodeForNodeIdx { idx })
+        self.nodes
+            .get_mut(idx.0)
+            .ok_or(TreeError::NoNodeForNodeIdx { idx })
     }
 
     pub fn new_node<P>(&mut self, parent: P) -> TreeResult<NodeIdx>
-    where P: Into<Option<NodeIdx>> {
+    where
+        P: Into<Option<NodeIdx>>,
+    {
         let data = D::default();
         if let Some(cidx) = self.garbage.pop_front() {
             self.node_mut(cidx)?.parent = parent.into();
@@ -54,7 +65,12 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
         } else {
             let parent: Option<NodeIdx> = parent.into();
             let cidx = NodeIdx(self.nodes.len());
-            self.nodes.push(Node { idx: cidx, parent, children: vec![], data });
+            self.nodes.push(Node {
+                idx: cidx,
+                parent,
+                children: vec![],
+                data,
+            });
             if let Some(pidx) = parent {
                 self.node_mut(pidx)?.add_child(cidx);
             }
@@ -67,7 +83,9 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
         if let Some(pidx) = self.node_ref(idx)?.parent {
             // Filter out the NodeIdx from the parent's child indices
             let mut parent = self.node_mut(pidx)?;
-            parent.children = parent.children.drain(..)
+            parent.children = parent
+                .children
+                .drain(..)
                 .filter(|&child_idx| child_idx != idx)
                 .collect();
         }
@@ -82,11 +100,7 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
     /// This means that the root of the `subtree` becomes a child of
     /// the node in `self` @ `idx`.  Note that the `NodeIdx`s of any
     /// and all nodes in `subtree` will *NOT* be valid in `self`.
-    pub fn add_subtree(
-        &mut self,
-        dst_node_idx: NodeIdx,
-        src: &Self
-    ) -> TreeResult<()> {
+    pub fn add_subtree(&mut self, dst_node_idx: NodeIdx, src: &Self) -> TreeResult<()> {
         type SrcTreeIdx = Option<NodeIdx>;
         type DstTreeIdx = NodeIdx;
         let mut map = HashMap::<SrcTreeIdx, DstTreeIdx>::new();
@@ -137,10 +151,7 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
 
     /// Return an iterator over the nodes, in DFS order,
     /// of the subtree rooted in `start`.
-    pub fn dfs(
-        &self,
-        start: NodeIdx
-    ) -> TreeResult<impl DoubleEndedIterator<Item = NodeIdx>> {
+    pub fn dfs(&self, start: NodeIdx) -> TreeResult<impl DoubleEndedIterator<Item = NodeIdx>> {
         let mut output: Vec<NodeIdx> = Vec::with_capacity(self.nodes.len());
         let mut stack: Vec<NodeIdx> = vec![start];
         while let Some(idx) = stack.pop() {
@@ -153,10 +164,7 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
     #[allow(unused)]
     /// Return an iterator, in BFS order, over the `NodeIdx`s
     /// of the nodes of the the subtree rooted in `start`.
-    pub fn bfs(
-        &self,
-        start: NodeIdx
-    ) -> TreeResult<impl DoubleEndedIterator<Item = NodeIdx>> {
+    pub fn bfs(&self, start: NodeIdx) -> TreeResult<impl DoubleEndedIterator<Item = NodeIdx>> {
         type Layer = Vec<NodeIdx>;
         let start_layer: Layer = vec![start];
         let mut layers: Vec<Layer> = vec![start_layer];
@@ -165,7 +173,9 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
             for &idx in previous.iter() {
                 current.extend(self.node_ref(idx)?.children());
             }
-            if current.is_empty() { break }
+            if current.is_empty() {
+                break;
+            }
             layers.push(current);
             current = vec![];
         }
@@ -173,7 +183,10 @@ impl<D> ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
     }
 }
 
-impl<D> fmt::Display for ArenaTree<D> where D: Clone + Debug + Default + PartialEq {
+impl<D> fmt::Display for ArenaTree<D>
+where
+    D: Clone + Debug + Default + PartialEq,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         // NOTE: This loop is `O(D * N)`, where:
         //       - D is the maximum depth of `self`
@@ -183,38 +196,47 @@ impl<D> fmt::Display for ArenaTree<D> where D: Clone + Debug + Default + Partial
             let num_ancestors: usize = self.ancestors_of(node_idx)
                 .unwrap(/*TreeResult*/)
                 .count();
-            for _ in 0 .. num_ancestors { write!(f, "| ")?; /* no newline */ }
+            for _ in 0..num_ancestors {
+                write!(f, "| ")?; /* no newline */
+            }
             writeln!(f, "{} {:?}", node.idx, node.data)?;
         }
         Ok(())
     }
 }
 
-
 impl<D: Serialize> Serialize for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq + Serialize {
+where
+    D: Clone + Debug + Default + PartialEq + Serialize,
+{
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
         const NUM_FIELDS: usize = 2;
         let mut state = serializer.serialize_struct("ArenaTree", NUM_FIELDS)?;
-        state.serialize_field("nodes",   &self.nodes)?;
+        state.serialize_field("nodes", &self.nodes)?;
         state.serialize_field("garbage", &self.garbage)?;
         state.end()
     }
 }
 
 impl<'de, D> Deserialize<'de> for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq + Deserialize<'de> {
+where
+    D: Clone + Debug + Default + PartialEq + Deserialize<'de>,
+{
     fn deserialize<DE: Deserializer<'de>>(d: DE) -> Result<Self, DE::Error> {
-
         #[derive(Deserialize)]
         #[serde(field_identifier, rename_all = "lowercase")]
-        enum Field { Nodes, Garbage }
+        enum Field {
+            Nodes,
+            Garbage,
+        }
 
         #[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
         struct ArenaTreeVisitor<D>(std::marker::PhantomData<D>);
 
         impl<'de, D> Visitor<'de> for ArenaTreeVisitor<D>
-        where D: Clone + Debug + Default + PartialEq + Deserialize<'de> {
+        where
+            D: Clone + Debug + Default + PartialEq + Deserialize<'de>,
+        {
             type Value = ArenaTree<D>;
 
             fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -222,16 +244,22 @@ where D: Clone + Debug + Default + PartialEq + Deserialize<'de> {
             }
 
             fn visit_seq<V>(self, mut seq: V) -> Result<Self::Value, V::Error>
-            where V: SeqAccess<'de> {
-                let nodes = seq.next_element()?
+            where
+                V: SeqAccess<'de>,
+            {
+                let nodes = seq
+                    .next_element()?
                     .ok_or_else(|| de::Error::invalid_length(0, &self))?;
-                let garbage = seq.next_element()?
+                let garbage = seq
+                    .next_element()?
                     .ok_or_else(|| de::Error::invalid_length(1, &self))?;
                 Ok(ArenaTree { nodes, garbage })
             }
 
             fn visit_map<A>(self, mut map: A) -> Result<Self::Value, A::Error>
-            where A: MapAccess<'de> {
+            where
+                A: MapAccess<'de>,
+            {
                 let mut nodes = None;
                 let mut garbage = None;
                 while let Some(key) = map.next_key()? {
@@ -241,20 +269,18 @@ where D: Clone + Debug + Default + PartialEq + Deserialize<'de> {
                                 return Err(de::Error::duplicate_field("nodes"));
                             }
                             nodes = Some(map.next_value()?);
-                        },
+                        }
                         Field::Garbage => {
                             if garbage.is_some() {
                                 return Err(de::Error::duplicate_field("garbage"));
                             }
                             garbage = Some(map.next_value()?);
-                        },
+                        }
                     }
                 }
                 Ok(ArenaTree {
-                    nodes: nodes
-                        .ok_or_else(|| de::Error::missing_field("nodes"))?,
-                    garbage: garbage
-                        .ok_or_else(|| de::Error::missing_field("garbage"))?,
+                    nodes: nodes.ok_or_else(|| de::Error::missing_field("nodes"))?,
+                    garbage: garbage.ok_or_else(|| de::Error::missing_field("garbage"))?,
                 })
             }
         }
@@ -263,20 +289,16 @@ where D: Clone + Debug + Default + PartialEq + Deserialize<'de> {
     }
 }
 
-
 impl<D> Core for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq
-    + for<'de> Deserialize<'de> + Serialize
-    + Core
+where
+    D: Clone + Debug + Default + PartialEq + for<'de> Deserialize<'de> + Serialize + Core,
 {
     type Delta = ArenaTreeDelta<D>;
 }
 
-
 impl<D> Apply for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq
-    + for<'de> Deserialize<'de> + Serialize
-    + Apply
+where
+    D: Clone + Debug + Default + PartialEq + for<'de> Deserialize<'de> + Serialize + Apply,
 {
     fn apply(&self, delta: Self::Delta) -> DeltaResult<Self> {
         match delta.0 {
@@ -289,9 +311,8 @@ where D: Clone + Debug + Default + PartialEq
 }
 
 impl<D> Delta for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq
-    + for<'de> Deserialize<'de> + Serialize
-    + Delta
+where
+    D: Clone + Debug + Default + PartialEq + for<'de> Deserialize<'de> + Serialize + Delta,
 {
     fn delta(&self, rhs: &Self) -> DeltaResult<Self::Delta> {
         Ok(ArenaTreeDelta(if self == rhs {
@@ -303,9 +324,8 @@ where D: Clone + Debug + Default + PartialEq
 }
 
 impl<D> FromDelta for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq
-    + for<'de> Deserialize<'de> + Serialize
-    + FromDelta
+where
+    D: Clone + Debug + Default + PartialEq + for<'de> Deserialize<'de> + Serialize + FromDelta,
 {
     fn from_delta(delta: Self::Delta) -> DeltaResult<Self> {
         match delta.0 {
@@ -318,25 +338,22 @@ where D: Clone + Debug + Default + PartialEq
 }
 
 impl<D> IntoDelta for ArenaTree<D>
-where D: Clone + Debug + Default + PartialEq
-    + for<'de> Deserialize<'de> + Serialize
-    + IntoDelta
+where
+    D: Clone + Debug + Default + PartialEq + for<'de> Deserialize<'de> + Serialize + IntoDelta,
 {
     fn into_delta(self) -> DeltaResult<Self::Delta> {
         Ok(ArenaTreeDelta(Some(self)))
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-#[derive(serde_derive::Serialize, serde_derive::Deserialize)]
+#[derive(Clone, Debug, PartialEq, serde_derive::Serialize, serde_derive::Deserialize)]
 pub struct ArenaTreeDelta<D>(Option<ArenaTree<D>>)
-where D: Clone + Debug + Default + PartialEq + Core;
+where
+    D: Clone + Debug + Default + PartialEq + Core;
 
-
-
-
-#[derive(Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
-    #[derive(deltoid_derive::Delta, Deserialize, Serialize)]
+#[derive(
+    Clone, PartialEq, Eq, PartialOrd, Ord, Hash, deltoid_derive::Delta, Deserialize, Serialize,
+)]
 pub struct Node<D> {
     idx: NodeIdx,
     parent: Option<NodeIdx>,
@@ -346,7 +363,9 @@ pub struct Node<D> {
 
 impl<D> Node<D> {
     #[inline(always)]
-    pub fn parent(&self) -> Option<NodeIdx> { self.parent }
+    pub fn parent(&self) -> Option<NodeIdx> {
+        self.parent
+    }
 
     #[inline(always)]
     pub fn set_parent(&mut self, parent: NodeIdx) {
@@ -354,13 +373,13 @@ impl<D> Node<D> {
     }
 
     #[inline(always)]
-    pub fn children<'n>(
-        &'n self
-    ) -> impl DoubleEndedIterator<Item = NodeIdx> + 'n {
+    pub fn children<'n>(&'n self) -> impl DoubleEndedIterator<Item = NodeIdx> + 'n {
         self.children.iter().map(|&idx| idx)
     }
 
-    pub fn count_children(&self) -> usize { self.children.len() }
+    pub fn count_children(&self) -> usize {
+        self.children.len()
+    }
 
     #[inline(always)]
     pub fn add_child(&mut self, child: NodeIdx) {
@@ -368,13 +387,33 @@ impl<D> Node<D> {
     }
 
     #[inline(always)]
-    pub fn node_idx(&self) -> NodeIdx { self.idx }
+    pub fn node_idx(&self) -> NodeIdx {
+        self.idx
+    }
 
     #[inline(always)]
-    pub fn data_ref(&self) -> &D { &self.data }
+    pub fn data_ref(&self) -> &D {
+        &self.data
+    }
 
     #[inline(always)]
-    pub fn data_mut(&mut self) -> &mut D { &mut self.data }
+    pub fn data_mut(&mut self) -> &mut D {
+        &mut self.data
+    }
+}
+
+impl<D> std::ops::Deref for Node<D> {
+    type Target = D;
+
+    fn deref(&self) -> &Self::Target {
+        &self.data
+    }
+}
+
+impl<D> std::ops::DerefMut for Node<D> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.data
+    }
 }
 
 impl<D: Debug> fmt::Debug for Node<D> {
@@ -392,9 +431,9 @@ impl<D: Debug> fmt::Debug for Node<D> {
     }
 }
 
-
-#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
-#[derive(deltoid_derive::Delta, Deserialize, Serialize)]
+#[derive(
+    Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, deltoid_derive::Delta, Deserialize, Serialize,
+)]
 pub struct NodeIdx(pub(crate) usize);
 
 impl NodeIdx {
@@ -417,12 +456,11 @@ impl std::ops::Add<usize> for NodeIdx {
     type Output = Self;
 
     #[inline(always)]
-    fn add(self, rhs: usize) -> Self { Self(self.0 + rhs) }
+    fn add(self, rhs: usize) -> Self {
+        Self(self.0 + rhs)
+    }
 }
 
-
-
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -454,47 +492,13 @@ mod tests {
             root,
             ancestors_order: vec![node20, node2, root],
             bfs_order: vec![
-                root,
-                node0,
-                node1,
-                node2,
-                node00,
-                node01,
-                node10,
-                node20,
-                node21,
-                node200
+                root, node0, node1, node2, node00, node01, node10, node20, node21, node200,
             ],
             dfs_order: vec![
-                root,
-                node0,
-                node00,
-                node01,
-                node1,
-                node10,
-                node2,
-                node20,
-                node200,
-                node21,
-            ],
-            subtree_bfs_order: vec![
-                root,
-                node1,
-                node2,
-                node10,
-                node20,
-                node21,
-                node200
-            ],
-            subtree_dfs_order: vec![
-                root,
-                node1,
-                node10,
-                node2,
-                node20,
-                node200,
-                node21,
+                root, node0, node00, node01, node1, node10, node2, node20, node200, node21,
             ],
+            subtree_bfs_order: vec![root, node1, node2, node10, node20, node21, node200],
+            subtree_dfs_order: vec![root, node1, node10, node2, node20, node200, node21],
         })
     }
 
@@ -514,7 +518,6 @@ mod tests {
         Ok(())
     }
 
-
     #[test]
     fn add_subtree() -> TreeResult<()> {
         let mut tree: ArenaTree<String> = ArenaTree::new();
@@ -562,20 +565,20 @@ mod tests {
         tree.add_subtree(node20, &subtree)?;
 
         let mut expected: ArenaTree<String> = ArenaTree::new();
-        let e_root: NodeIdx = expected.new_node(None)?;                //  0
-        let e_node0: NodeIdx = expected.new_node(e_root)?;             //  1
-        let e_node1: NodeIdx = expected.new_node(e_root)?;             //  2
-        let e_node10: NodeIdx = expected.new_node(e_node1)?;           //  3
-        let e_node100: NodeIdx = expected.new_node(e_node10)?;         //  4
-        let e_node2: NodeIdx = expected.new_node(e_root)?;             //  5
-        let e_node20: NodeIdx = expected.new_node(e_node2)?;           //  6
-        let e_st_root: NodeIdx = expected.new_node(e_node20)?;         //  7
-        let e_st_node0: NodeIdx = expected.new_node(e_st_root)?;       //  8
-        let e_st_node1: NodeIdx = expected.new_node(e_st_root)?;       //  9
-        let e_st_node10: NodeIdx = expected.new_node(e_st_node1)?;     // 10
-        let e_st_node100: NodeIdx = expected.new_node(e_st_node10)?;   // 11
-        let e_st_node2: NodeIdx = expected.new_node(e_st_root)?;       // 12
-        let e_st_node20: NodeIdx = expected.new_node(e_st_node2)?;     // 13
+        let e_root: NodeIdx = expected.new_node(None)?; //  0
+        let e_node0: NodeIdx = expected.new_node(e_root)?; //  1
+        let e_node1: NodeIdx = expected.new_node(e_root)?; //  2
+        let e_node10: NodeIdx = expected.new_node(e_node1)?; //  3
+        let e_node100: NodeIdx = expected.new_node(e_node10)?; //  4
+        let e_node2: NodeIdx = expected.new_node(e_root)?; //  5
+        let e_node20: NodeIdx = expected.new_node(e_node2)?; //  6
+        let e_st_root: NodeIdx = expected.new_node(e_node20)?; //  7
+        let e_st_node0: NodeIdx = expected.new_node(e_st_root)?; //  8
+        let e_st_node1: NodeIdx = expected.new_node(e_st_root)?; //  9
+        let e_st_node10: NodeIdx = expected.new_node(e_st_node1)?; // 10
+        let e_st_node100: NodeIdx = expected.new_node(e_st_node10)?; // 11
+        let e_st_node2: NodeIdx = expected.new_node(e_st_root)?; // 12
+        let e_st_node20: NodeIdx = expected.new_node(e_st_node2)?; // 13
         let expected_pairs = [
             (e_root, "root"),
             (e_node0, "node0"),
@@ -620,5 +623,4 @@ mod tests {
         assert_eq!(ancestors, data.ancestors_order);
         Ok(())
     }
-
 }
